/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.iit.genomics.cru.isomirage;

import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.JDialog;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.SwingWorker;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

/**
 * @(#)AnalysisRunner.java
 * 11 Sept 2014
 * Copyright 2014 Computational Research Unit of IIT@SEMM. All rights reserved.
 * Use is subject to MIT license terms.
 * 
 * Swing worker class executing analyses.
 * 
 * @author Heiko Muller
 * @version 1.0
 * @since 1.0
 */
public class AnalysisRunner  extends SwingWorker{
    
    /**
     * Reference to the graphical user interface
     */
    Isomirage gui;
    
    /**
     * Constructor for AnalysisRunner
     * @param gui 
     */
    public AnalysisRunner(Isomirage gui){
        this.gui = gui;
    }
    
    /**
     * Runs Isomirage analysis.
     * @return Object
     * @throws Exception 
     */
    @Override
    protected Object doInBackground() throws Exception {
        //throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
        ArrayList<String> mapfiles = gui.getMapfiles();
        String genome = gui.getGenome();
        gui.initProgressBar(0, mapfiles.size() + 3);
        JTextArea progressreport = gui.getjTextArea3();
        progressreport.setText("Started analysis...\r\n");
        JDialog dialog = gui.getjDialog1();
        JTextPane jTextPane1 = gui.getjTextPane1();
        JTable jTable1 = gui.getjTable1();
        JTable jTable2 = gui.getjTable2();
        JTable jTable3 = gui.getjTable3();
        JTable jTable4 = gui.getjTable4();
        JTable jTable5 = gui.getjTable5();
        JTable jTable6 = gui.getjTable6();
        int progress = 0;
        for(String s : mapfiles){
            progressreport.append("analyzing : " + s + "...");
            progress++;
            gui.setProgressBarValue(progress);
            CountIsoforms ci = new CountIsoforms(genome, s);
            String msg = "";
            try{
                msg = (String)ci.doInBackground();
            }catch(Exception e){
            
            }
            if(msg.length() > 0){
                dialog.setTitle("Incompatible genome");
                dialog.toFront();
                jTextPane1.setText("The following sequence wasn't found in the genome you chose:\r\n" + msg);
                
                dialog.setPreferredSize(new Dimension(200, 100));
                dialog.setVisible(true);
                System.out.println("incompatible genome");
                return null;
            }
            progressreport.append("done\r\n");
        }
        progressreport.append("creating raw data table...");
        CountIsoforms.parse(mapfiles);
        progressreport.append("done\r\n");
        
        //progressreport.append("creating RMA normalized data table...");
        //RMA rma = new RMA(mapfiles.get(0) + "_linked.txt");
        //progressreport.append("done\r\n");
        
        progressreport.append("creating RMA summary data table...");    
        RMASummary rmasummary = new RMASummary(mapfiles.get(0) + "_linked.txt");
        progressreport.append("done\r\n");   
        
        //progressreport.append("creating RPKM normalized data table...");        
        //RPKM rpkm = new RPKM(mapfiles.get(0) + "_linked.txt");
        //progressreport.append("done\r\n");
        
        progressreport.append("creating RPKM summary data table...");
        RPKMSummary rpkmsummary = new RPKMSummary(mapfiles.get(0) + "_linked.txt");
        progressreport.append("done\r\n");
        
        progress++;
        gui.setProgressBarValue(progress);
        progressreport.append("loading raw data table...");
        setDataTable(mapfiles.get(0) + "_linked.txt_raw_data.txt", jTable1);
        progressreport.append("done\r\n");
        progress++;
        gui.setProgressBarValue(progress);
        progressreport.append("loading RMA normalized data table...");
        setDataTable(mapfiles.get(0) + "_linked.txt_RMA.txt", jTable2);
        progressreport.append("done\r\n");
        progress++;
        gui.setProgressBarValue(progress);
        progressreport.append("loading RPKM normalized data table...");
        setDataTable(mapfiles.get(0) + "_linked.txt_RPKM.txt", jTable3);
        progressreport.append("done\r\n");
        
        progressreport.append("loading raw summary data table...");
        setDataTable(mapfiles.get(0) + "_linked.txt_raw_summary.txt", jTable5);
        progressreport.append("done\r\n");
        
        progressreport.append("loading RMA summary data table...");
        setDataTable(mapfiles.get(0) + "_linked.txt_RMA_summary.txt", jTable6);
        progressreport.append("done\r\n");
        
        progressreport.append("loading RPKM summary data table...");
        setDataTable(mapfiles.get(0) + "_linked.txt_RPKM_summary.txt", jTable4);
        progressreport.append("done\r\n");
        
        
        
        gui.setProgressBarValue(0);
        progressreport.append("\r\n");
        
        progressreport.append("raw data file: " + mapfiles.get(0) + "_linked.txt_raw_data.txt\r\n");
        progressreport.append("RMA normalized data file: " + mapfiles.get(0) + "_linked.txt_RMA.txt\r\n");
        progressreport.append("RPKM normalized data file: " + mapfiles.get(0) + "_linked.txt_RPKM.txt\r\n");
        
        progressreport.append("raw summary file: " + mapfiles.get(0) + "_linked.txt_raw_summary.txt\r\n");
        progressreport.append("RMA summary file: " + mapfiles.get(0) + "_linked.txt_RMA_summary.txt\r\n");
        progressreport.append("RPKM summary file: " + mapfiles.get(0) + "_linked.txt_RPKM_summary.txt\r\n");
        
        progressreport.append("Analysis complete.");
        return null;
    }
    
    /**
     * Setter for reference to graphical user interface
     * @param gui 
     */
    public void setGui(Isomirage gui){
        this.gui = gui;
    }
    
    /**
     * Reads a file
     * @param filename
     * @return ArrayList<String>
     */
    private ArrayList<String> readTable(String filename){
        ArrayList<String> result = new ArrayList<String>();
        try{
            FileReader fr = new FileReader(filename);
            BufferedReader br = new BufferedReader(fr);
            String line = "";
            while( (line = br.readLine()) != null && (line.trim().length() > 0) ){
                result.add(line);
            }
        }catch(IOException ioe){
            ioe.printStackTrace();
        }
        return result;
    }
    
    /**
     * Displays file content in a JTable
     * @param file
     * @param table 
     */
    public void setDataTable(String file, JTable table){            
        String[][] data = parseTable(readTable(file));
        table.setModel(new DefaultTableModel(data, data[0]));
        TableModel tm = table.getModel(); 
        for(int i = 0; i < data.length; i++){
            for(int j = 0; j < data[0].length; j++){
                tm.setValueAt(data[i][j], i, j);
            }
        }
    }
    
    /**
     * Parses ArrayList of Strings as a table
     * @param table
     * @return String[][]
     */
    private String[][] parseTable(ArrayList<String> table){        
        String[][] result = new String[Math.min(300000, table.size())][1];
        for(int i = 0; i < result.length; i++){
            result[i] = table.get(i).split("\t");
        }        
        return result;
    }
}
