/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.iit.genomics.cru.isomirage;

import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;

/**
 * @(#)RMA.java
 * 11 Sept 2014
 * Copyright 2014 Computational Research Unit of IIT@SEMM. All rights reserved.
 * Use is subject to MIT license terms.
 * 
 * Performs RMA normalization of occurrence counts.
 * 
 * @author Heiko Muller
 * @version 1.0
 * @since 1.0
 */
public class RMA {
    
    /**
     * row ids of the data table
     */
    String[] ids = null;
    
    /**
     * data of the data table
     */
    double[][] inputdata = null;//col 1 == data value sample 1, col 2 == data value sample 2 etc
    
    /**
     * data of the data table
     */
    double[][] data = null;//col 1 == index sample 1, col 2 == data value sample 1, col 3 == index sample 2, col 4 == data value sample 2 etc
    
    /**
     * sorted data table, holds individually sorted data columns
     */
    double[][] sorted_data = null;//col 1 == index sample 1, col 2 == data value sample 1, col 3 == index sample 2, col 4 == data value sample 2 etc
    
    /**
     * normalized data table
     */
    double[][] normalized_data = null;//col 1 == index sample 1, col 2 == data value sample 1, col 3 == index sample 2, col 4 == data value sample 2 etc
    
    /**
     * reverse sorted data table
     */
    double[][] unsorted_data = null;//col 1 == index sample 1, col 2 == data value sample 1, col 3 == index sample 2, col 4 == data value sample 2 etc
    
    /**
     * header
     */
    String header = "";
    

    /**
     * Constructor. Filename points to input table with row headers and without column headers.
     * @param filename 
     */
    public RMA(String filename){
        readFile(filename);
        generateSortedData();
        generateNormalizedData();
        generateUnsortedData();
        writeOutput(filename);
    }

    /**
     * Reads input data, inits and fills data table
     * @param filename 
     */
    private void readFile(String filename){
        int rows = readFileLineNumber(filename) - 1;//minus header line
        int current = 0;
        ids = new String[rows];
        try{
            FileReader fr = new FileReader(filename);
            BufferedReader br = new BufferedReader(fr);
            header = br.readLine();
            String line = "";
            line = br.readLine();
            String[] temp = line.split("\t");
            inputdata = new double[rows][temp.length - 1];
            data = new double[rows][2 * (temp.length - 1)];
            sorted_data = new double[rows][2 * (temp.length - 1)];
            normalized_data = new double[rows][2 * (temp.length - 1)];
            unsorted_data = new double[rows][2 * (temp.length - 1)];
            ids[current] = temp[0];
            for(int i = 1; i < temp.length; i++){
                data[current][2 * (i - 1)] = current;
                data[current][2 * (i - 1) + 1] = Double.parseDouble(temp[i]);
                inputdata[current][i - 1] = Double.parseDouble(temp[i]);

            }
            while((line = br.readLine()) != null && line.trim().length() > 0){
                current++;
                //System.out.println(current);
                temp = line.split("\t");
                ids[current] = temp[0];
                for(int i = 1; i < temp.length; i++){
                    data[current][2 * (i - 1)] = current;
                    data[current][2 * (i - 1) + 1] = Double.parseDouble(temp[i]);
                    inputdata[current][i - 1] = Double.parseDouble(temp[i]);
                }
            }

        fr.close();
        br.close();
        }catch(IOException ioe){
        }
    }

    /**
     * Reads the number of lines in input file
     * @param filename
     * @return int line number
     */
     private int readFileLineNumber(String filename){
         int result = 0;
        try{
            FileReader fr = new FileReader(filename);
            BufferedReader br = new BufferedReader(fr);
            String line = "";
            while((line = br.readLine()) != null && line.trim().length() > 0){
                result++;
            }
        }catch(IOException ioe){}
        return result;
     }

     /**
      * sorts data table
      */
     private void generateSortedData(){
         Sort s = new Sort();         
         for(int i = 0; i < data[0].length - 1; i = i + 2){
             data = s.shellsort(data, i + 1);
            for(int j = 0; j < data.length; j++){
                sorted_data[j][i] = data[j][i];
                sorted_data[j][i + 1] = data[j][i + 1];
            }
         }
     }
     
     /**
      * averages a row
      * @param d
      * @return double
      */
     private double average(double[] d){
        double result = 0;
        for(int i = 0; i < d.length - 1; i = i + 2){
            result = result + d[i + 1];
        }
        result = result/(d.length/2);
        return result;
     }

     /**
      * Generates normalized data
      */
     private void generateNormalizedData(){
         for(int j = 0; j < data.length; j++){
             double avg = average(sorted_data[j]);
            for(int i = 0; i < data[0].length - 1; i = i + 2){
                normalized_data[j][i] = sorted_data[j][i];
                normalized_data[j][i + 1] = avg;
            }
        }
     }

     /**
      * Unsorts data
      */
     private void generateUnsortedData(){

         Sort s = new Sort();
         for(int i = 0; i < normalized_data[0].length - 1; i = i + 2){
             normalized_data = s.shellsort(normalized_data, i);
            for(int j = 0; j < normalized_data.length; j++){
                unsorted_data[j][i] = normalized_data[j][i];
                unsorted_data[j][i + 1] = normalized_data[j][i + 1];
            }
         }
     }

     /**
      * Writes normalized data output to file.
      * Output file is located in the directory of the input file and has an _RMA.txt extension.
      * @param filename 
      */
     private void writeOutput(String filename){
        try{
            FileWriter fwr = new FileWriter(filename + "_RMA.txt");
            fwr.write(header + "\r\n");
            for(int j = 0; j < data.length; j++){
                if(sum(extractData(unsorted_data[j])) > 0){
                    fwr.write(ids[j] + "\t");
                    for(int i = 0; i < data[0].length - 1; i = i + 2){
                        fwr.write(unsorted_data[j][i + 1] + "\t");
                    }
                    fwr.write("\r\n");
                }
            }
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){

        }
     }
     
     private double[] extractData(double[] d){
         double[] result = new double[d.length/2];
         for(int i = 0; i < result.length; i++){
            result[i] =  d[2*i + 1];
         }
         return result;
     }

     private double sum(double[] d){
        double result = 0;
        for(int i = 0; i < d.length; i++){
            result = result + d[i];
        }
        return result;
    }
     
     protected void writeDataOutput(String filename){
        try{
            FileWriter fwr = new FileWriter(filename + "_raw_data.txt");
            fwr.write(header + "\r\n");
            for(int j = 0; j < inputdata.length; j++){
                    if(sum(inputdata[j]) > 0){
                        fwr.write(ids[j] + "\t");
                        for(int i = 0; i < inputdata[0].length; i++){
                            fwr.write(inputdata[j][i] + "\t");
                        }
                        fwr.write("\r\n");
                    }
                
            }
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){

        }
     }

}
