/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.iit.genomics.cru.isomirage;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;

/**
 *
 * @author hmuller
 */
public class RMASummary extends RMA{
    
    Hashtable<String, double[]> summarydata;
    double[][] rawdata;
    double[][] normdata;
    
    public RMASummary(String filename){
        super(filename);     
        //System.out.println("extracting RMA data");
        rawdata = inputdata;
        normdata = extractData(unsorted_data);
        //dumpRawData();
        writeDataOutput(filename);
        //System.out.println("adding RMA group data");
        addRMAGroupData();
        //System.out.println("writing RMA group data");
        writeRMASummaryOutput(filename);
        //System.out.println("adding raw group data");
        addRawGroupData();
        //System.out.println("writing raw group data");
        writeRawSummaryOutput(filename);
    }
    
    private double[][] extractData(double[][] input){
        double[][] result = new double[input.length][input[0].length/2];
        for(int i = 0; i < result.length; i++){
            for(int j = 0; j < result[0].length; j++){
                result[i][j] = input[i][2*j + 1];
            }
        }
        return result;
    }
    
    private void addRMAGroupData(){
        summarydata = new Hashtable<String, double[]>();
        for(int i = 0; i < normdata.length; i++){
            //System.out.println("summary calculation " + i);
            IsoformNameParser inp = new IsoformNameParser(ids[i]);
            String group = inp.getGroup();
            String extension = inp.getExtension();
            String templated = inp.getTemplated();
            String mirnaname = inp.getMirnaname();
            
            String groupkey = mirnaname + "_" + group + "_" + getExtensionGroup(extension);
            if(groupkey.endsWith("_")){
                groupkey = groupkey.substring(0, groupkey.length() - 1);
            }
            String templatekey = mirnaname + "_" + group + "_" + templated;
            
            if(templatekey.endsWith("_")){
                templatekey = templatekey.substring(0, templatekey.length() - 1);
            }
            //System.out.println(groupkey + " " + templatekey);
            if(templated.equals("nt")){
                if(summarydata.containsKey(groupkey)){
                    double[] temp = summarydata.get(groupkey);
                    for(int j = 0; j < temp.length; j++){
                        temp[j] = temp[j] + normdata[i][j];
                    }
                }else{
                    double[] temp = new double[normdata[i].length];
                    for(int j = 0; j < temp.length; j++){
                        temp[j] = temp[j] + normdata[i][j];
                    }
                    summarydata.put(groupkey, temp);
                }
            }
            
            if(summarydata.containsKey(templatekey)){
                double[] temp = summarydata.get(templatekey);
                for(int j = 0; j < temp.length; j++){
                    temp[j] = temp[j] + normdata[i][j];
                }
            }else{
                double[] temp = new double[normdata[i].length];
                for(int j = 0; j < temp.length; j++){
                    temp[j] = temp[j] + normdata[i][j];
                }
                summarydata.put(templatekey, temp);
            }
        }
    }
    
    private void addRawGroupData(){
        summarydata = new Hashtable<String, double[]>();
        for(int i = 0; i < rawdata.length; i++){
            //System.out.println("summary calculation " + i);
            IsoformNameParser inp = new IsoformNameParser(ids[i]);
            String group = inp.getGroup();
            String extension = inp.getExtension();
            String templated = inp.getTemplated();
            String mirnaname = inp.getMirnaname();
            
            String groupkey = mirnaname + "_" + group + "_" + getExtensionGroup(extension);
            if(groupkey.endsWith("_")){
                groupkey = groupkey.substring(0, groupkey.length() - 1);
            }
            String templatekey = mirnaname + "_" + group + "_" + templated;
            if(templatekey.endsWith("_")){
                templatekey = templatekey.substring(0, templatekey.length() - 1);
            }
            //System.out.println(groupkey + " " + templatekey);
            
            if(templated.equals("nt")){
                if(summarydata.containsKey(groupkey)){
                    double[] temp = summarydata.get(groupkey);
                    for(int j = 0; j < temp.length; j++){
                        temp[j] = temp[j] + rawdata[i][j];
                    }
                }else{
                    double[] temp = new double[rawdata[i].length];
                    for(int j = 0; j < temp.length; j++){
                        temp[j] = temp[j] + rawdata[i][j];
                    }
                    summarydata.put(groupkey, temp);
                }
            }
            
            if(summarydata.containsKey(templatekey)){
                double[] temp = summarydata.get(templatekey);
                for(int j = 0; j < temp.length; j++){
                    temp[j] = temp[j] + rawdata[i][j];
                }
            }else{
                double[] temp = new double[rawdata[i].length];
                for(int j = 0; j < temp.length; j++){
                    temp[j] = temp[j] + rawdata[i][j];
                }
                summarydata.put(templatekey, temp);
            }
        }
    }
    
    private String getExtensionGroup(String extension){
        if(extension.length() == 0){
            return "";
        }
        if(extension.equals("A") || extension.equals("AA") || extension.equals("AAA")){
            return "A-form";
        }
        if(extension.equals("C") || extension.equals("CC") || extension.equals("CCC")){
            return "C-form";
        }
        if(extension.equals("G") || extension.equals("GG") || extension.equals("GGG")){
            return "G-form";
        }
        if(extension.equals("T") || extension.equals("TT") || extension.equals("TTT")){
            return "U-form";
        }        
        return "mixed";
    }
    
    /**
      * Writes normalized data output to file.
      * Output file is located in the directory of the input file and has an _RPKM.txt extension.
      * @param filename 
      */
    private void writeRMASummaryOutput(String filename){
        try{
            FileWriter fwr = new FileWriter(filename + "_RMA_summary.txt");
            fwr.write(header + "\r\n");
            Object[] keys = summarydata.keySet().toArray();
            for(int j = 0; j < keys.length; j++){
                double[] d = summarydata.get(keys[j]);
                if(sum(d) > 0){
                    fwr.write((String)keys[j] + "\t");
                    for(int i = 0; i < d.length; i++){
                        fwr.write(d[i] + "\t");
                    }
                    fwr.write("\r\n");
                }
            }
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){

        }
     }
    
    /**
      * Writes normalized data output to file.
      * Output file is located in the directory of the input file and has an _RPKM.txt extension.
      * @param filename 
      */
    private void writeRawSummaryOutput(String filename){
        try{
            FileWriter fwr = new FileWriter(filename + "_raw_summary.txt");
            fwr.write(header + "\r\n");
            Object[] keys = summarydata.keySet().toArray();
            for(int j = 0; j < keys.length; j++){
                double[] d = summarydata.get(keys[j]);
                if(sum(d) > 0){
                    fwr.write((String)keys[j] + "\t");
                    for(int i = 0; i < d.length; i++){
                        fwr.write(d[i] + "\t");
                    }
                    fwr.write("\r\n");
                }
            }
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){

        }
     }
    
    private double sum(double[] d){
        double result = 0;
        for(int i = 0; i < d.length; i++){
            result = result + d[i];
        }
        return result;
    }
    
    private void dumpRawData(){
        for(int i = 0; i < rawdata.length && i < 10000; i++){
            System.out.print(ids[i] + "\t");
            for(int j = 0; j < rawdata[0].length; j++){
                System.out.print(rawdata[i][j] + "\t");
            }
            System.out.println("");
        }
    }
}
