/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.iit.genomics.cru.isomirage;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;

/**
 *
 * @author hmuller
 */
public class RPKMSummary extends RPKM {
    
    Hashtable<String, double[]> summarydata;
    
    public RPKMSummary(String filename){
        super(filename);
        summarydata = new Hashtable<String, double[]>();
        addGroupData();
        writeSummaryOutput(filename);
        
    }
    
    private void addGroupData(){
        for(int i = 0; i < normalized_data.length; i++){
            //System.out.println("summary calculation " + i);
            IsoformNameParser inp = new IsoformNameParser(ids[i]);
            String group = inp.getGroup();
            String extension = inp.getExtension();
            String templated = inp.getTemplated();
            String mirnaname = inp.getMirnaname();
            
            String groupkey = mirnaname + "_" + group + "_" + getExtensionGroup(extension);
            if(groupkey.endsWith("_")){
                groupkey = groupkey.substring(0, groupkey.length() - 1);
            }
            String templatekey = mirnaname + "_" + group + "_" + templated;
            if(templatekey.endsWith("_")){
                templatekey = templatekey.substring(0, templatekey.length() - 1);
            }
            //System.out.println(groupkey + " " + templatekey);
            
            if(templated.equals("nt")){
                if(summarydata.containsKey(groupkey)){
                    double[] data = summarydata.get(groupkey);
                    for(int j = 0; j < data.length; j++){
                        data[j] = data[j] + normalized_data[i][j];
                    }
                }else{
                    double[] data = new double[normalized_data[i].length];
                    for(int j = 0; j < data.length; j++){
                        data[j] = data[j] + normalized_data[i][j];
                    }
                    summarydata.put(groupkey, data);
                }
            }
            
            if(summarydata.containsKey(templatekey)){
                double[] data = summarydata.get(templatekey);
                for(int j = 0; j < data.length; j++){
                    data[j] = data[j] + normalized_data[i][j];
                }
            }else{
                double[] data = new double[normalized_data[i].length];
                for(int j = 0; j < data.length; j++){
                    data[j] = data[j] + normalized_data[i][j];
                }
                summarydata.put(templatekey, data);
            }
        }
    }
    
    private String getExtensionGroup(String extension){
        if(extension.length() == 0){
            return "";
        }
        if(extension.equals("A") || extension.equals("AA") || extension.equals("AAA")){
            return "A-form";
        }
        if(extension.equals("C") || extension.equals("CC") || extension.equals("CCC")){
            return "C-form";
        }
        if(extension.equals("G") || extension.equals("GG") || extension.equals("GGG")){
            return "G-form";
        }
        if(extension.equals("T") || extension.equals("TT") || extension.equals("TTT")){
            return "U-form";
        }        
        return "mixed";
    }
    
    /**
      * Writes normalized data output to file.
      * Output file is located in the directory of the input file and has an _RPKM.txt extension.
      * @param filename 
      */
    private void writeSummaryOutput(String filename){
        try{
            FileWriter fwr = new FileWriter(filename + "_RPKM_summary.txt");
            fwr.write(header + "\r\n");
            Object[] keys = summarydata.keySet().toArray();
            for(int j = 0; j < keys.length; j++){
                double[] data = summarydata.get(keys[j]);
                if(sum(data) > 0){
                    fwr.write((String)keys[j] + "\t");
                    for(int i = 0; i < data.length; i++){
                        fwr.write(data[i] + "\t");
                    }
                    fwr.write("\r\n");
                }
            }
            fwr.flush();
            fwr.close();
        }catch(IOException ioe){

        }
     }
    
    private double sum(double[] d){
        double result = 0;
        for(int i = 0; i < d.length; i++){
            result = result + d[i];
        }
        return result;
    }
    
    
    
    
    
}
